<?php

namespace EventPortal\Tests\Feature\Admin\Event;

use EventPortal\Domain\Admin\Models\Admin;
use EventPortal\Domain\Approval\Models\Approval;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Team\Models\Team;
use EventPortal\Domain\Permission\Models\Permission;
use EventPortal\Tests\Support\Traits\InteractsWithTeams;
use EventPortal\Tests\Support\Traits\SeedRolesAndPermissions;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\DatabaseMigrations;
use Illuminate\Http\Response;
use Spatie\Snapshots\MatchesSnapshots;

class AwaitingComplianceApprovalTest extends TestCase
{
    use DatabaseMigrations, MatchesSnapshots, InteractsWithTeams;

    /** @test **/
    public function can_view_awaiting_compliance_approval_index_and_see_events_awaiting_compliance_approval_only()
    {
        $team = factory(Team::class)->create();
        $adminTeam = factory(Team::class)->create();
        $admin = factory(Admin::class)->create();
        $event = factory(Event::class)->create(['name' => 'NOT IN TEAM']);
        $adminEventRequiringComplianceApproval = factory(Event::class)->create(['name' => 'LINE MANAGER APPROVED']);
        $adminEventApproved = factory(Event::class)->create(['name' => 'APPROVED']);
        $adminEventDeclined = factory(Event::class)->create(['name' => 'DECLINED']);

        $this->addToSameTeam(
            [$admin, $adminEventRequiringComplianceApproval, $adminEventApproved, $adminEventDeclined],
            $adminTeam
        );
        $this->addToSameTeam([$event], $team);

        factory(Approval::class)->create(['event_id' => $event->id, 'status' => 'line-manager-approved']);
        factory(Approval::class)->create([
            'event_id' => $adminEventRequiringComplianceApproval->id,
            'status' => 'line-manager-approved',
        ]);
        factory(Approval::class)->create(['event_id' => $adminEventApproved->id, 'status' => 'approved']);
        factory(Approval::class)->create(['event_id' => $adminEventDeclined->id, 'status' => 'declined']);

        $permission = factory(Permission::class)->create([
            'name' => 'event.awaiting-compliance-approval',
        ]);
        $admin->givePermissionTo($permission);

        $this->assertMatchesJsonSnapshot(
            $this->actingAsAdmin($admin)->get(route('admin.event.awaiting-compliance-approval'))->props()
        );
    }

    /** @test **/
    public function cannot_view_compliance_approvals_without_permission()
    {
        $event = factory(Event::class)->create(['name' => 'Hello World!']);
        factory(Approval::class)->create(['event_id' => $event->id, 'status' => 'line-manager-approved']);
        $admin = factory(Admin::class)->create();

        factory(Permission::class)->create([
            'name' => 'event.awaiting-compliance-approval',
        ]);

        $this->actingAsAdmin($admin)
            ->get(route('admin.event.awaiting-compliance-approval'))
            ->assertStatus(Response::HTTP_UNAUTHORIZED);
    }

    /** @test **/
    public function can_filter_to_view_other_event_types()
    {
        $adminTeam = factory(Team::class)->create();
        $admin = factory(Admin::class)->create();
        $adminEventRequiringComplianceApproval = factory(Event::class)->create(['name' => 'LINE MANAGER APPROVED']);
        $adminEventApproved = factory(Event::class)->create(['name' => 'APPROVED']);
        $adminEventDeclined = factory(Event::class)->create(['name' => 'DECLINED']);

        $this->addToSameTeam(
            [$admin, $adminEventRequiringComplianceApproval, $adminEventApproved, $adminEventDeclined],
            $adminTeam
        );

        factory(Approval::class)->create([
            'event_id' => $adminEventRequiringComplianceApproval->id,
            'status' => 'line-manager-approved',
        ]);
        factory(Approval::class)->create(['event_id' => $adminEventApproved->id, 'status' => 'approved']);
        factory(Approval::class)->create(['event_id' => $adminEventDeclined->id, 'status' => 'declined']);

        $permission = factory(Permission::class)->create([
            'name' => 'event.awaiting-compliance-approval',
        ]);
        $admin->givePermissionTo($permission);

        $this->assertMatchesJsonSnapshot(
            $this->actingAsAdmin($admin)
                ->get(route('admin.event.awaiting-compliance-approval', ['filter' => ['status' => 'approved']]))
                ->props()
        );
    }
}
